@@ -1,8 +1,10 @@
+from uuid import uuid4
+
 import pytest
 from qrcode import QRCode
 
 from pynubank import MockHttpClient
-from pynubank.exception import NuMissingCreditCard
+from pynubank.exception import NuMissingCreditCard, NuInvalidAuthenticationMethod
 from pynubank.nubank import Nubank
 from pynubank.utils.graphql import prepare_request_body
 from pynubank.utils.mock_http import GHOSTFLAME_URL
@@ -219,7 +221,7 @@ def test_get_card_statements():
 
 def test_get_account_balance():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     balance = nubank_client.get_account_balance()
 
@@ -228,7 +230,7 @@ def test_get_account_balance():
 
 def test_get_account_feed():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     statements = nubank_client.get_account_feed()
 
@@ -251,7 +253,7 @@ def test_get_account_feed():
 
 def test_get_account_statements():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     statements = nubank_client.get_account_statements()
 
@@ -273,7 +275,7 @@ def test_get_account_statements():
 
 def test_get_account_investments_details():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     statements = nubank_client.get_account_investments_details()
 
@@ -305,7 +307,7 @@ def test_get_account_investments_details():
 
 def test_get_account_investments_yield():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     month_yield = nubank_client.get_account_investments_yield()
 
@@ -342,21 +344,21 @@ def test_get_qr_code(monkeypatch):
 
 def test_should_generate_boleto():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     assert nubank_client.create_boleto(200.50) == '123131321231231.2313212312.2131231.21332123'
 
 
 def test_should_create_money_request():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     assert nubank_client.create_money_request(200) == 'https://some.tld/path1/path2'
 
 
 def test_should_fetch_pix_keys():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     data = nubank_client.get_available_pix_keys()
 
@@ -370,7 +372,7 @@ def test_should_fetch_pix_keys():
 
 def test_should_create_pix_money_request():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     keys_data = nubank_client.get_available_pix_keys()
     request = nubank_client.create_pix_payment_qrcode('1231231232', 1232213.23, keys_data['keys'][0])
@@ -401,7 +403,7 @@ def test_should_return_none_if_isnt_pix_transaction():
     client.remove_mock_url((GHOSTFLAME_URL, str(prepare_request_body('pix_receipt_screen'))))
     client.add_mock_url(GHOSTFLAME_URL, str(prepare_request_body('pix_receipt_screen')), 'pix_receipt_screen_not_found')
     nubank_client = Nubank(client)
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     pix_identifier = nubank_client.get_pix_identifier('tx_123123')
 
@@ -414,7 +416,7 @@ def test_should_return_none_if_pix_transaction_doesnt_have_identifier():
     client.add_mock_url(GHOSTFLAME_URL, str(prepare_request_body('pix_receipt_screen')),
                         'pix_receipt_screen_without_identifier')
     nubank_client = Nubank(client)
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     pix_identifier = nubank_client.get_pix_identifier('tx_123123')
 
@@ -423,8 +425,100 @@ def test_should_return_none_if_pix_transaction_doesnt_have_identifier():
 
 def test_should_retrieve_pix_identifier_for_pix_transaction():
     nubank_client = Nubank(client=MockHttpClient())
-    nubank_client.authenticate_with_qr_code('12345678912', 'hunter12', 'some-uuid')
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
 
     pix_identifier = nubank_client.get_pix_identifier('tx_123123')
 
     assert pix_identifier == 'IdentificadorPixAqui'
+
+
+def test_get_card_statement_details():
+    statement_mock = {
+        '_links': {
+            'self': {
+                'href': f'https://mocked-proxy-url/api/transactions/{uuid4()}'
+            }
+        }
+    }
+
+    nubank_client = Nubank(client=MockHttpClient())
+    nubank_client.authenticate_with_cert('1234', 'hunter12', 'some-file.p12')
+    statement_details = nubank_client.get_card_statement_details(statement_mock)
+
+    transaction = statement_details['transaction']
+    assert transaction['category'] == "outros"
+    assert transaction['amount'] == 10000
+    assert transaction['card_last_four_digits'] == '1234'
+    assert transaction['charges'] == 2
+    assert transaction['original_merchant_name'] == 'Loja'
+    assert transaction['charges_list'] == [
+        {
+            "amount": 5000,
+            "status": "future",
+            "index": 1,
+            "source": "installments_merchant",
+            "extras": [],
+            "post_date": "2021-09-10"
+        },
+        {
+            "amount": 5000,
+            "status": "future",
+            "index": 2,
+            "source": "installments_merchant",
+            "extras": [],
+            "post_date": "2021-10-10"
+        }
+    ]
+
+
+@pytest.mark.parametrize('method_name,method_args', [
+    ('get_account_balance', {}),
+    ('get_account_statements', {}),
+    ('get_account_feed', {}),
+    ('get_account_investments_yield', {}),
+    ('get_account_statements', {}),
+    ('create_boleto', {'amount': 100}),
+    ('create_money_request', {'amount': 100}),
+    ('get_available_pix_keys', {}),
+    ('create_pix_payment_qrcode', {'account_id': '2', 'amount': 123, 'pix_key': {}}),
+    ('get_pix_identifier', {'transaction_id': 'abc123'}),
+])
+def test_nuconta_methods_should_fail_with_web_authentication(method_name, method_args):
+    nu = Nubank(client=MockHttpClient())
+    nu.authenticate_with_qr_code('1234', 'hunter12', 'uuid')
+
+    method = getattr(nu, method_name)
+    with pytest.raises(NuInvalidAuthenticationMethod):
+        method(**method_args)
+
+
+@pytest.mark.parametrize('method_name,method_args', [
+    ('get_card_feed', {}),
+    ('get_card_statements', {}),
+    ('get_card_payments', {}),
+    ('get_bills', {}),
+    ('get_customer', {}),
+    ('get_bill_details', {'bill': {
+        '_links': {
+            'self': {
+                'href': 'https://mocked-proxy-url/api/bills/abcde-fghi-jklmn-opqrst-uvxz'
+            }
+        }
+    }}),
+    ('get_card_statement_details', {'statement': {
+        '_links': {
+            'self': {'href': f'https://mocked-proxy-url/api/transactions/{uuid4()}'}
+        }
+    }}),
+])
+def test_creditcard_methods_should_allow_web_authentication(method_name, method_args):
+    nu = Nubank(client=MockHttpClient())
+    nu.authenticate_with_qr_code('1234', 'hunter12', 'uuid')
+
+    method = getattr(nu, method_name)
+
+    method(**method_args)
+
+
+def test_creditcard_methods_should_allow_app_authentication():
+    pass
